/*
Copyright 2002 - 2004 UnitedAdmins
Project originally created by OLO (http://olo.counter-strike.pl)

This file is part of StatsMe.

   StatsMe is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   StatsMe is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with StatsMe; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   A copy of the GNU General Public License is available in a file called COPYING.
   See README for more information.

*/

/* $Id: engine_api.cpp,v 1.16 2004/03/21 01:01:43 punisher Exp $ */

#include <stdio.h>
#include <stdarg.h>

#include <extdll.h>

#include "engine_api.h"
#include "meta_api.h"
#include "statsme.h"

int	gmsgCurWeapon=0;
int	gmsgDeathMsg;
int	gmsgDamage;
int	gmsgTextMsg;
int	gmsgTeamInfo;
int	gmsgWeaponList;
int	gmsgResetHUD;
int	gmsgintermission;
int gmsgMOTD;
int gmsgServerName;
int gmsgHealth;
int gmsgAmmoX;
int gmsgBattery;
int gmsgShowMenu;
int gmsgSendAudio;
int gmsgTeamScore;
int gmsgMoney;
int	gmsgBleeding;
int	gmsgBandaging;
int gmsgScreenFade;
int gmsgScoreInfo;
//int	gmsgInitHUD;

fun_t modMsgsEnd[MAX_REG_MSGS];
fun_t modMsgs[MAX_REG_MSGS];

fun_call_t*	modMsgsFunCall[MAX_SM_REG_MSGS];
void (*function)(void*);
void (*endfunction)(void*);
player_t* mPlayer;
int mPlayerIndex;
int	mState;

#ifdef DEBUG_OPTION
static FILE*fp;
#endif

void MessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed) {
	if (ed){
		mPlayerIndex = ENTINDEX(ed);
		mPlayer = GET_PLAYER_POINTER_I(mPlayerIndex);
	} else {
		mPlayerIndex = 0;
		mPlayer = NULL;
	}
#ifdef DEBUG_OPTION
	if ( sm_debug->value )	{
		int size;
		fp = fopen( sm.debug_name,"a");
		fprintf(fp,"pfnMessageBegin: dest=%d type=%s(%d)\n",mPlayerIndex,GET_USER_MSG_NAME(PLID, msg_type, &size),msg_type);
	}
#endif
	mState = 0;
	function=modMsgs[msg_type].function;
	endfunction=modMsgsEnd[msg_type].function;
	PARSE_MESSAGE_BEGIN(msg_type, mPlayer ,mPlayerIndex);
	RETURN_META(MRES_IGNORED);
}

void MessageEnd(void) {
	PARSE_MESSAGE_END();
#ifdef DEBUG_OPTION
	if ( sm_debug->value )	{
		fprintf(fp,"pfnMessageEnd\n\n");
		fclose(fp);  
	}
#endif
	if (endfunction) (*endfunction)(NULL);
	RETURN_META(MRES_IGNORED);
}

void WriteByte(int iValue) {
	PARSE_WRITE_BYTE(iValue);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteByte=%d\n",iValue);
#endif
	if (function)	(*function)((void *)&iValue);
	RETURN_META(MRES_IGNORED);
}
void WriteChar(int iValue) {
	PARSE_WRITE_CHAR(iValue);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteChar=%c\n",iValue);
#endif
	if (function)	(*function)((void *)&iValue);
	RETURN_META(MRES_IGNORED);
}
void WriteShort(int iValue) {
	PARSE_WRITE_SHORT(iValue);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteShort=%d\n",iValue);
#endif
	if (function)	(*function)((void *)&iValue);
	RETURN_META(MRES_IGNORED);
}
void WriteLong(int iValue) {
	PARSE_WRITE_LONG(iValue);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteLong=%d\n",iValue);
#endif
	if (function)	(*function)((void *)&iValue);
	RETURN_META(MRES_IGNORED);
}
void WriteAngle(float flValue) {
	PARSE_WRITE_ANGLE(flValue);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteAngle=%.4f\n",flValue);
#endif
	if (function) (*function)((void *)&flValue);
	RETURN_META(MRES_IGNORED);
}
void WriteCoord(float flValue) {
	PARSE_WRITE_COORD(flValue);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteCoord=%.4f\n",flValue);
#endif
	if (function)	(*function)((void *)&flValue);
	RETURN_META(MRES_IGNORED);
}
void WriteString(const char *sz) {
	PARSE_WRITE_STRING(sz);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteString=\"%s\"\n",sz);
#endif
	if (function)	(*function)((void *)sz);
	RETURN_META(MRES_IGNORED);
}
void WriteEntity(int iValue) {
	PARSE_WRITE_ENTITY(iValue);
#ifdef DEBUG_OPTION
	if ( sm_debug->value ) fprintf(fp,"pfnWriteEntity=%d\n",iValue);
#endif
	if (function)	(*function)((void *)&iValue);
	RETURN_META(MRES_IGNORED);
}

void TraceLine(const float *v1, const float *v2, int fNoMonsters, edict_t *pentToSkip, TraceResult *ptr) {
	TRACE_LINE(v1, v2, fNoMonsters, pentToSkip, ptr);
	if (ptr->iHitgroup && pentToSkip && pentToSkip->v.flags & player_bits) {
		player_t* pPlayer = GET_PLAYER_POINTER(pentToSkip);
		if (pPlayer->weapon) pPlayer->weapon->dist = (pentToSkip->v.origin - ptr->vecEndPos).Length();
		pPlayer->aiming_at = ptr->iHitgroup; //save this part of body
	}
	RETURN_META(MRES_SUPERCEDE);
}

enginefuncs_t meta_engfuncs;
C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion ) {
	meta_engfuncs.pfnMessageBegin = MessageBegin;
	meta_engfuncs.pfnMessageEnd = MessageEnd;
	meta_engfuncs.pfnWriteByte = WriteByte;
	meta_engfuncs.pfnWriteChar = WriteChar;
	meta_engfuncs.pfnWriteShort = WriteShort;
	meta_engfuncs.pfnWriteLong = WriteLong;
	meta_engfuncs.pfnWriteCoord = WriteCoord;
	meta_engfuncs.pfnWriteAngle = WriteAngle;
	meta_engfuncs.pfnWriteString = WriteString;
	meta_engfuncs.pfnWriteEntity =  WriteEntity;
	meta_engfuncs.pfnTraceLine = TraceLine;

	switch ( sm.mod_id )
	{
		case MOD_CSTRIKE:
			meta_engfuncs.pfnEmitSound = EmitSound_CS;
			meta_engfuncs.pfnSetModel = SetModel_CS;
			break;
		case MOD_DOD:		
			meta_engfuncs.pfnSetModel = SetModel_DOD;
			break;
		case MOD_TFC:
			// meta_engfuncs.pfnSetModel = SetModel_TFC;
			break;
	}

	memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));
	return TRUE;
}

engtask_t*	sm_engtask;
fun_call_t*	parseFun;
bool parseAMCall;
bool parseNotDone;
char parseMsg[768];
int	 parseLen;
int  parsePos;
msg_vault_t parseVault[MAX_PARSE_VALUES]; // engine message values

void PARSE_MESSAGE_BEGIN(int msg_type, player_t *target, int index) {

	parseNotDone = parseAMCall = false;
	if ( (parseFun = modMsgsFunCall[msg_type]) != NULL ){ //am plugins calls which are registered with sm_register
		for(fun_call_t*p=parseFun;p;p=p->next){ //go through all and check if they are for specified player or for world msg
			if ( p->flag && !(sm.sm_mode[p->mode] & p->flag) ) { //do we have set flag?
				p->done = true;
				continue;
			}
			if ( sm_enabled->value == 0 ) { // was statsme disabled by menu ?
			    p->done = true;
				continue;
			}
			if ( target ) { // is this for specified player - check also if should be alive or dead 
				if ( !p->player || ( (target->pEdict->v.deadflag == DEAD_NO) ? !p->alive : !p->dead  ) ) {
					p->done = true;
					continue;
				}
			}
			else if ( !p->world ){
				p->done = true;
				continue;
			}
			if ( p->once && p->stamp == (float)gpGlobals->time ){ //have to be executed only once?
				p->done = true;
				continue;
			}
			if ( !p->fillme ) parseAMCall = true; // AM Plugin - we must prepare copy of message as server command
			parseNotDone = true; // we have got something to check
		}
		if ( parseNotDone ) {
			sm.player = target;
			if ( parseAMCall ) parseLen=sprintf(&parseMsg[64]," %d",index) + 64; //start printing first data which is index of recivier
			parseVault[parsePos = 0].type = V_INTEGER;
			parseVault[parsePos].iValue = index;
		}
	}
}

void SET_MESSAGE_TASK(const char* cmd, int len, int flag, int id)
{
	engtask_t* a = new engtask_t;
	a->cmd = new char[len];
	strcpy(a->cmd,cmd);
	a->flag = flag;
	a->id = id;
	a->done = false;
	a->next = NULL;
	if (sm_engtask){
		engtask_t*b = sm_engtask;
		while (b->next)
			b = b->next;
		b->next = a;
	}
	else {
		sm_engtask = a;
	}
}

void PARSE_MESSAGE_END(void) {
	if ( parseAMCall ) {
		parseMsg[parseLen++] = ';'; 
		parseMsg[parseLen++] = 0;
	}
	for ( fun_call_t*p = parseFun ; p ; p = p->next ) {
		if (p->done) {
			p->done = false; //unset this msg but don't call it
			continue;
		}
		if ( p->fillme ) {
			char* todo = smv_putvars( p->cmd );
			if ( todo && *todo )
				SET_MESSAGE_TASK( todo, strlen(todo) + 1,p->flag,p->mode );
		}
		else {
			strncpy(&parseMsg[p->where],p->cmd,p->len);
			SET_MESSAGE_TASK(&parseMsg[p->where], parseLen ,p->flag,p->mode);
		}
		p->stamp = gpGlobals->time; //save when last done
	}
}

void PARSE_INTEGER(int iValue) {
	if ( parseNotDone ) {
		if ( parseAMCall ) parseLen += sprintf(&parseMsg[parseLen]," %d", iValue );
		parseVault[++parsePos].type = V_INTEGER;
		parseVault[parsePos].iValue = iValue;
		for (fun_call_t*p=parseFun;p;p=p->next){ // go through values and check for conditiones
			if ( p->done || !p->cond[parsePos] ) continue; // no conditions or already done
			bool skip = true;
			for ( cond_t* a = p->cond[parsePos] ; a ; a = a->next ) {
				skip=false;
				if ( a->type == '=' )	{
					if ( a->iValue == iValue ){
						skip=true;
						break;
					}
				}
				else if ( a->type == '!' )	{
					if ( a->iValue != iValue ){
						skip=true;
						break;
					}
				}
				else if ( a->type == '&' )	{
					if ( iValue & a->iValue ){
						skip=true;
						break;
					}
				}
				else if ( a->type  == '<' ) {
					if ( iValue < a->iValue  ) {
						skip=true;
						break;
					}
				}
				else if ( a->type  == '>' ) {
					if ( iValue > a->iValue  ) {
						skip=true;
						break;
					}
				}
			}
			if (skip) continue;
			p->done = true;
		}
	}
}

void PARSE_FLOAT(float flValue) {
	if ( parseNotDone ) {
		if ( parseAMCall ) parseLen += sprintf(&parseMsg[parseLen]," %.4f",flValue);
		parseVault[++parsePos].type = V_FLOAT;
		parseVault[parsePos].fValue = flValue;
		for (fun_call_t*p=parseFun;p;p=p->next){ // go through values and check for conditiones
			if ( p->done || !p->cond[parsePos] ) continue; // no conditions or already done
			bool skip = true;
			for ( cond_t* a = p->cond[parsePos] ; a ; a = a->next ) {
				skip=false;
				if ( a->type == '=' )	{
					if ( a->fValue == flValue ) {
						skip=true;
						break;
					}
				}
				else if ( a->type == '!' )	{
					if ( a->fValue != flValue ) {
						skip=true;
						break;
					}
				}
				else if ( a->type  == '<' ) {
					if ( flValue < a->fValue  ) {
						skip=true;
						break;
					}
				}
				else if ( a->type  == '>' ) {
					if ( flValue > a->fValue ) {
						skip=true;
						break;
					}
				}
			}
			if (skip) continue;
			p->done = true;
		}
	}
}

void PARSE_STRING(const char *sz) {
	if ( parseNotDone ){
		if ( parseAMCall ) parseLen += sprintf(&parseMsg[parseLen],parseFun->quotes ? " \"%s\"" : " %s",sz);
		parseVault[++parsePos].type = V_STRING;
		parseVault[parsePos].sValue = sz;
		for (fun_call_t*p=parseFun;p;p=p->next){ // go through values and check for conditiones
			if ( p->done || !p->cond[parsePos] ) continue; // no conditions or already done
			bool skip = true;
			for (cond_t* a = p->cond[parsePos] ; a ; a = a->next ) {
				skip=false;
				if ( a->type == '=' )	{
					if (p->whole ? !strcmp( sz, a->sValue) : !strncmp( sz ,a->sValue ,a->len) ){
						skip=true;
						break;
					}
				}
				else if ( a->type == '!' )	{
					if (p->whole ? strcmp( sz , a->sValue): strncmp( sz ,a->sValue ,a->len) ){
						skip=true;
						break;
					}
				}
				else if ( a->type == '&' )	{
					if ( strstr( sz ,a->sValue) )	{
						skip=true;
						break;
					}
				}
			}
			if (skip) continue;
			p->done = true;
		}
	}
}
